@@ -50,76 +50,6 @@ typedef struct {
 }
 iq_png_t;
 
-/*  _____                         
-   | ____|_ __ _ __ ___  _ __ ___ 
-   |  _| | '__| '__/ _ \| '__/ __|
-   | |___| |  | | | (_) | |  \__ \
-   |_____|_|  |_|  \___/|_|  |___/ */
-                               
-
-static struct liq_error_c {
-    int number;
-    char * message;
-}
-liq_error_stuff[] = {
-    {LIQ_VALUE_OUT_OF_RANGE, "value out of range"},
-    {LIQ_OUT_OF_MEMORY, "out of memory"},
-    {LIQ_NOT_READY, "not ready"},
-    {LIQ_BITMAP_NOT_AVAILABLE, "bitmap not available"},
-    {LIQ_BUFFER_TOO_SMALL, "buffer too small"},
-    {LIQ_INVALID_POINTER, "invalid pointer"},
-    {LIQ_HISTOGRAM_FAILED, "get_histogram failed"},
-    {LIQ_FREED_POINTER, "bad access to freed pointer"},
-    {LIQ_NULL_POINTER, "null pointer"},
-    {LIQ_BAD_POINTER, "pointer with wrong type"},
-};
-
-#define N_LIQ_ERRORS (sizeof (liq_error_stuff) / sizeof (struct liq_error_c))
-
-static char * image_quantize_get_liq_error_string (int error)
-{
-    int i;
-    for (i = 0; i < N_LIQ_ERRORS; i++) {
-	if (error == liq_error_stuff[i].number) {
-	    return liq_error_stuff[i].message;
-	}
-    }
-    return "undocumented error code";
-}
-
-#define MAX_ERROR_MSG 0x1000
-
-static void
-image_quantize_error (image_quantize_t * iq, liq_error error,
-		      char * msg, ...)
-{
-    char error_message[MAX_ERROR_MSG];
-    char * liq_error_string;
-    int used;
-
-    liq_error_string = image_quantize_get_liq_error_string (error);
-
-    used = 0;
-    used += snprintf (error_message + used, MAX_ERROR_MSG - used,
-		      "libimagequant error %d (%s) with input ",
-		      error, liq_error_string);
-    va_list args;
-    va_start (args, msg);
-    vsnprintf (error_message + used, MAX_ERROR_MSG - used,
-	       msg, args);
-    va_end (args);
-    croak (error_message);
-}
-
-static void
-image_quantize_perl_error (image_quantize_t * iq, char * msg, ...)
-{
-    va_list args;
-    va_start (args, msg);
-    vcroak (msg, & args);
-    va_end (args);
-}
-
 /*  __  __                                 
    |  \/  | ___ _ __ ___   ___  _ __ _   _ 
    | |\/| |/ _ \ '_ ` _ \ / _ \| '__| | | |
@@ -159,10 +89,10 @@ image_quantize_free_image (image_quantize_t * iq)
     }
 }
 
-/* Free everything and check we have the right number of things left. */
+/* Free everything except "iq" itself. */
 
 static void
-image_quantize_free (image_quantize_t * iq)
+image_quantize_free_allocated (image_quantize_t * iq)
 {
     image_quantize_free_image (iq);
     if (iq->attr) {
@@ -170,6 +100,15 @@ image_quantize_free (image_quantize_t * iq)
 	IQFREE;
 	iq->attr = 0;
     }
+}
+
+/* Free everything and check we have the right number of things left. */
+
+static void
+image_quantize_free (image_quantize_t * iq)
+{
+    image_quantize_free_allocated (iq);
+
     if (iq->n_mallocs != 0) {
 	warn ("Image::Quantize: There are %d pieces of unfreed memory",
 	      iq->n_mallocs);
@@ -177,6 +116,117 @@ image_quantize_free (image_quantize_t * iq)
     Safefree (iq);
 }
 
+/*  _____                         
+   | ____|_ __ _ __ ___  _ __ ___ 
+   |  _| | '__| '__/ _ \| '__/ __|
+   | |___| |  | | | (_) | |  \__ \
+   |_____|_|  |_|  \___/|_|  |___/ */
+                               
+
+static struct liq_error_c {
+    int number;
+    char * message;
+}
+liq_error_stuff[] = {
+    {LIQ_VALUE_OUT_OF_RANGE, "value out of range"},
+    {LIQ_OUT_OF_MEMORY, "out of memory"},
+    {LIQ_NOT_READY, "not ready"},
+    {LIQ_BITMAP_NOT_AVAILABLE, "bitmap not available"},
+    {LIQ_BUFFER_TOO_SMALL, "buffer too small"},
+    {LIQ_INVALID_POINTER, "invalid pointer"},
+    {LIQ_HISTOGRAM_FAILED, "get_histogram failed"},
+    {LIQ_FREED_POINTER, "bad access to freed pointer"},
+    {LIQ_NULL_POINTER, "null pointer"},
+    {LIQ_BAD_POINTER, "pointer with wrong type"},
+    {LIQ_NEGATIVE_DIMENSIONS, "image dimensions are negative"},
+    {LIQ_HUGE_DIMENSIONS, "image dimensions are unfeasible"},
+    {LIQ_BAD_USER_POINTER, "liq hates your pointer"},
+};
+
+#define N_LIQ_ERRORS (sizeof (liq_error_stuff) / sizeof (struct liq_error_c))
+
+static char * image_quantize_get_liq_error_string (int error)
+{
+    int i;
+    for (i = 0; i < N_LIQ_ERRORS; i++) {
+	if (error == liq_error_stuff[i].number) {
+	    return liq_error_stuff[i].message;
+	}
+    }
+    return "undocumented error code";
+}
+
+#define MAX_ERROR_MSG 0x1000
+
+static void
+image_quantize_error (image_quantize_t * iq, liq_error error,
+		      char * msg, ...)
+{
+    char error_message[MAX_ERROR_MSG];
+    char * liq_error_string;
+    int used;
+
+    image_quantize_free_image (iq);
+
+    liq_error_string = image_quantize_get_liq_error_string (error);
+
+    used = 0;
+    used += snprintf (error_message + used, MAX_ERROR_MSG - used,
+		      "libimagequant error %d (%s) with input ",
+		      error, liq_error_string);
+    va_list args;
+    va_start (args, msg);
+    vsnprintf (error_message + used, MAX_ERROR_MSG - used,
+	       msg, args);
+    va_end (args);
+    croak (error_message);
+}
+
+static void
+image_quantize_perl_error (image_quantize_t * iq, char * msg, ...)
+{
+    va_list args;
+
+    image_quantize_free_image (iq);
+
+    va_start (args, msg);
+    vcroak (msg, & args);
+    va_end (args);
+}
+
+static void
+image_quantize_log_callback (const liq_attr * attr, const char * message,
+			     void * user_info)
+{
+    dSP;
+    SV * message_sv;
+    image_quantize_t * iq;
+
+    iq = user_info;
+
+    ENTER;
+    SAVETMPS;
+
+    /* Convert the C string "message" into a Perl SV. */
+
+    message_sv = sv_2mortal (newSVpv (message, 0));
+
+    /* Push our stuff onto the Perl stack. */
+
+    PUSHMARK (SP);
+    XPUSHs (message_sv);
+    XPUSHs (iq->data);
+    PUTBACK;
+
+    /* Call the callback in "iq->function", discarding its return
+       values. */
+
+    call_sv (iq->function, G_DISCARD);
+
+    FREETMPS;
+    LEAVE;
+}
+
 /*  ___                                 ___                    _   _         
    |_ _|_ __ ___   __ _  __ _  ___ _ _ / _ \ _   _  __ _ _ __ | |_(_)_______ 
     | || '_ ` _ \ / _` |/ _` |/ _ (_|_) | | | | | |/ _` | '_ \| __| |_  / _ \
@@ -212,6 +262,7 @@ image_quantize_png_data_to_rgba (image_quantize_t * iq)
     int bit_depth;
     iq_png_t * p;
     png_bytepp row_pointers;
+    liq_error error;
 
     p = iq->hook;
 
@@ -226,9 +277,12 @@ image_quantize_png_data_to_rgba (image_quantize_t * iq)
 				   colour_type);
     }
     row_pointers = png_get_rows (p->in_png, p->in_info);
-    iq->image =
-	liq_image_create_rgba_rows (iq->attr, (void **) row_pointers,
-				    iq->width, iq->height, 0);
+    error = liq_image_create_rgba_rows_2 (iq->attr, (void **) row_pointers,
+					  iq->width, iq->height, 0,
+					  & iq->image);
+    if (error) {
+	image_quantize_error (iq, error, "error creating image");
+    }
     IQGET;
 }
 
@@ -325,32 +379,3 @@ image_quantize_png_quantize (image_quantize_t * iq)
     png_set_PLTE (p->out_png, p->out_info, png_colours, lp->count);
 }
 
-static void
-image_quantize_log_callback (const liq_attr * attr, const char * message,
-			     void * user_info)
-{
-    dSP;
-    SV * function;
-    SV * data;
-    SV * message_sv;
-    image_quantize_t * iq = user_info;
-
-    function = iq->function;
-    data = iq->data;
-
-    ENTER;
-    SAVETMPS;
-
-    message_sv = sv_2mortal (newSVpv (message, 0));
-
-    PUSHMARK (SP);
-    XPUSHs (message_sv);
-    XPUSHs (data);
-    PUTBACK;
-
-    call_sv (function, G_DISCARD);
-
-    FREETMPS;
-    LEAVE;
-}
-
