@@ -202,52 +202,33 @@ LIQ_EXPORT liq_error liq_invalid_any (void * any)
     return LIQ_OK;
 }
 
+#define INVALIDITYCHECK(any)				\
+    liq_error error;					\
+    error = liq_invalid_any (any);			\
+    if (error != LIQ_OK) {				\
+	return error;					\
+    }							\
+    if (any->magic_header == liq_freed_magic) {		\
+	return LIQ_FREED_POINTER;			\
+    }							\
+    if (any->magic_header != liq_ ## any ## _magic) {	\
+	return LIQ_INVALID_POINTER;			\
+    }							\
+    return LIQ_OK
+
 LIQ_EXPORT liq_error liq_invalid_attr (liq_attr * attr)
 {
-    liq_error error;
-    error = liq_invalid_any (attr);
-    if (error != LIQ_OK) {
-	return error;
-    }
-    if (attr->magic_header == liq_freed_magic) {
-	return LIQ_FREED_POINTER;
-    }
-    if (attr->magic_header != liq_attr_magic) {
-	return LIQ_INVALID_POINTER;
-    }
-    return LIQ_OK;
+    INVALIDITYCHECK(attr);
 }
 
 LIQ_EXPORT liq_error liq_invalid_image (liq_image * image)
 {
-    liq_error error;
-    error = liq_invalid_any (image);
-    if (error) {
-	return error;
-    }
-    if (image->magic_header == liq_freed_magic) {
-	return LIQ_FREED_POINTER;
-    }
-    if (image->magic_header != liq_image_magic) {
-	return LIQ_INVALID_POINTER;
-    }
-    return LIQ_OK;
+    INVALIDITYCHECK(image);
 }
 
 LIQ_EXPORT liq_error liq_invalid_result (liq_result * result)
 {
-    liq_error error;
-    error = liq_invalid_any (result);
-    if (error) {
-	return error;
-    }
-    if (result->magic_header == liq_freed_magic) {
-	return LIQ_FREED_POINTER;
-    }
-    if (result->magic_header != liq_result_magic) {
-	return LIQ_INVALID_POINTER;
-    }
-    return LIQ_OK;
+    INVALIDITYCHECK(result);
 }
 
 static double quality_to_mse(long quality)
@@ -499,6 +480,62 @@ static liq_image *liq_image_create_internal(liq_attr *attr, rgba_pixel* rows[],
     return img;
 }
 
+static liq_error liq_image_create_internal_2 (liq_attr *attr, rgba_pixel* rows[], liq_image_get_rgba_row_callback *row_callback, void *row_callback_user_info, int width, int height, double gamma, liq_image ** image_ptr)
+{
+    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
+	return LIQ_BAD_POINTER;
+    }
+    if (width <= 0 || height <= 0) {
+	return LIQ_NEGATIVE_DIMENSIONS;
+    }
+    if (gamma < 0 || gamma > 1.0) {
+	return LIQ_VALUE_OUT_OF_RANGE;
+    }
+
+    if (!rows && !row_callback) {
+	return LIQ_NULL_POINTER;
+    }
+    liq_image *img = attr->malloc(sizeof(liq_image));
+    if (!img) {
+	return LIQ_OUT_OF_MEMORY;
+    }
+    *img = (liq_image){
+        .magic_header = liq_image_magic,
+        .malloc = attr->malloc,
+        .free = attr->free,
+        .width = width, .height = height,
+        .gamma = gamma ? gamma : 0.45455,
+        .rows = rows,
+        .row_callback = row_callback,
+        .row_callback_user_info = row_callback_user_info,
+        .min_opaque_val = attr->min_opaque_val,
+    };
+
+    if (!rows || attr->min_opaque_val < 1.f) {
+        img->temp_row = attr->malloc(sizeof(img->temp_row[0]) * width * omp_get_max_threads());
+        if (!img->temp_row) {
+	    return LIQ_OUT_OF_MEMORY;
+	}
+    }
+
+    // if image is huge or converted pixels are not likely to be reused then don't cache converted pixels
+    if (liq_image_should_use_low_memory(img, !img->temp_row && !attr->use_contrast_maps && !attr->use_dither_map)) {
+        verbose_print(attr, "  conserving memory");
+        if (!liq_image_use_low_memory(img)) {
+	    /* This may be the wrong error to return, please check
+	       this. */
+	    return LIQ_OUT_OF_MEMORY;
+	}
+    }
+
+    if (img->min_opaque_val < 1.f) {
+        verbose_print(attr, "  Working around IE6 bug by making image less transparent...");
+    }
+
+    * image_ptr = img;
+    return LIQ_OK;
+}
+
 LIQ_EXPORT liq_error liq_image_set_memory_ownership(liq_image *img, int ownership_flags)
 {
     if (!CHECK_STRUCT_TYPE(img, liq_image)) return LIQ_INVALID_POINTER;
@@ -541,6 +578,30 @@ LIQ_EXPORT liq_image *liq_image_create_rgba_rows(liq_attr *attr, void* rows[], i
     return liq_image_create_internal(attr, (rgba_pixel**)rows, NULL, NULL, width, height, gamma);
 }
 
+LIQ_EXPORT liq_error liq_image_create_rgba_rows_2 (liq_attr *attr, void* rows[], int width, int height, double gamma, liq_image ** image_ptr)
+{
+    liq_error error;
+    * image_ptr = NULL;
+    if (width <= 0 || height <= 0) {
+	return LIQ_NEGATIVE_DIMENSIONS;
+    }
+    if (width > INT_MAX/height/16 || height > INT_MAX/width/16) {
+	return LIQ_HUGE_DIMENSIONS;
+    }
+    /* Commenting this check out, I don't think this is valid. See
+       "https://github.com/pornel/pngquant/issues/74". */
+
+      /*
+    for(int i=0; i < height; i++) {
+        if (!CHECK_USER_POINTER(rows+i) || !CHECK_USER_POINTER(rows[i])) {
+	    return LIQ_BAD_USER_POINTER;
+	}
+    }
+    */
+    error = liq_image_create_internal_2(attr, (rgba_pixel**)rows, NULL, NULL, width, height, gamma, image_ptr);
+    return error;
+}
+
 LIQ_EXPORT liq_image *liq_image_create_rgba(liq_attr *attr, void* bitmap, int width, int height, double gamma)
 {
     if (width <= 0 || height <= 0 || gamma < 0 || gamma > 1.0) return NULL;
@@ -933,8 +994,10 @@ static float remap_to_palette(liq_image *const input_image, unsigned char *const
     viter_state average_color[(VITER_CACHE_LINE_GAP+map->colors) * max_threads];
     viter_init(map, max_threads, average_color);
 
+#ifdef _OPENMP
     #pragma omp parallel for if (rows*cols > 3000) \
         default(none) shared(average_color) reduction(+:remapping_error)
+#endif /* def _OPENMP */
     for(int row = 0; row < rows; ++row) {
         const f_pixel *const row_pixels = liq_image_get_row_f(input_image, row);
         unsigned int last_match=0;
