@@ -47,6 +47,9 @@ typedef enum liq_error {
     LIQ_FREED_POINTER,
     LIQ_NULL_POINTER,
     LIQ_BAD_POINTER,
+    LIQ_NEGATIVE_DIMENSIONS,
+    LIQ_HUGE_DIMENSIONS,
+    LIQ_BAD_USER_POINTER,
 } liq_error;
 
 enum liq_ownership {LIQ_OWN_ROWS=4, LIQ_OWN_PIXELS=8};
@@ -78,6 +81,7 @@ LIQ_EXPORT void liq_set_log_callback(liq_attr*, liq_log_callback_function*, void
 LIQ_EXPORT void liq_set_log_flush_callback(liq_attr*, liq_log_flush_callback_function*, void* user_info);
 
 LIQ_EXPORT liq_image *liq_image_create_rgba_rows(liq_attr *attr, void* rows[], int width, int height, double gamma);
+LIQ_EXPORT liq_error liq_image_create_rgba_rows_2(liq_attr *attr, void* rows[], int width, int height, double gamma, liq_image ** image);
 LIQ_EXPORT liq_image *liq_image_create_rgba(liq_attr *attr, void* bitmap, int width, int height, double gamma);
 
 typedef void liq_image_get_rgba_row_callback(liq_color row_out[], int row, int width, void* user_info);
